==> gm-1.6.3_Linux.tar.gz: gm-1.6.3_Linux/tests/gm_time_registration.c <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation                      *
 * Copyright (c) 1996, 1997 by Myricom, Inc.                             *
 * All rights reserved.  See the file `COPYING' for copyright notice.    *
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "gm.h"


#define BUFFER_LEN	(2 * 1024 * 1024)
#define PORT	7


void msti_main(void);
void msti_main2(void);



int
main(int argc, char **argv)
{
	struct gm_port *p;
	gm_u64_t start, stop, start2, middle, total_reg, total_dereg;
	unsigned char *buffer, *startbuffer;
	gm_status_t status;
	unsigned int i, num_pages;
	float total_time, reg_time;
	int page_len = (int) getpagesize();
	int looping = 0;
	GM_PARAMETER_MAY_BE_UNUSED(argc);
	GM_PARAMETER_MAY_BE_UNUSED(argv);

	if (argc > 1) {
		looping = 1;
	}

	if (gm_open(&p, 0, PORT, "gm_time_reg",
				(enum gm_api_version) GM_API_VERSION)
		!= GM_SUCCESS)
	{
		printf("Couldn't open port 'p1' = %d\n", PORT);
		exit(-1);
	}

	startbuffer = gm_malloc(BUFFER_LEN + 2 * page_len);
	if (!startbuffer) {
		printf("Couldn't allocate a buffer\n");
		gm_close(p);
	}

	buffer = (unsigned char *) ((((unsigned long) startbuffer) + (page_len - 1)) & ~(page_len - 1));

	printf("Buffer allocated len=%d   start = %p  buffer = %p\n",
		   BUFFER_LEN, startbuffer, buffer);
	fflush(stdout);

	do {
		if (looping) {
			printf("looping\n");
		}

		printf("Registering\n");
		fflush(stdout);

		start = gm_ticks(p);

		for (i = 0; i < BUFFER_LEN; i += page_len) {
			status = gm_register_memory(p, buffer + i, page_len);
			if (status == GM_SUCCESS) {
				/* OK */
			}
			else {
				printf("Couldn't register memory offset = %d (0x%x)\n", i, i);
				gm_close(p);
				exit(-1);
			}
		}
		num_pages = i;

		middle = gm_ticks(p);

		printf("Deregistering\n");
		fflush(stdout);

		start2 = gm_ticks(p);


		for (i = 0; i < BUFFER_LEN; i += page_len) {
			status = gm_deregister_memory(p, buffer + i, page_len);
			if (status == GM_SUCCESS) {
				/* OK */
			}
			else {
				printf("Couldn't DEregister memory offset = %d (0x%x)\n", i, i);
				gm_close(p);
				exit(-1);
			}
		}

		stop = gm_ticks(p);

		total_reg = middle - start;
		total_dereg = stop - start2;

		num_pages = BUFFER_LEN / page_len;
		total_time = ((float) ((unsigned int) total_reg & (unsigned int) 0xFFFFFFFF)) / 2.0;
		printf("  Register time = "GM_U64_TMPL" ticks (%6.2f seconds)  %8.2f usecs/page\n",
			   GM_U64_ARG (total_reg),
			   total_time / 1000000.0, total_time / (float) num_pages);

		total_time = ((float) ((unsigned int) total_dereg & (unsigned int) 0xFFFFFFFF)) / 2.0;
		printf("Deregister time = "GM_U64_TMPL" ticks (%6.2f seconds)  %8.2f usecs/page\n\n\n",
			   GM_U64_ARG (total_dereg),
			   total_time / 1000000.0, total_time / (float) num_pages);

	} while (looping);

	gm_free(startbuffer);
	gm_close(p);

	printf("\nTimings when we register 16meg, then deregister 16 meg\n");
	msti_main();
	printf("\nTimings when we register size ITER times, then unregister size ITER times\n");
	msti_main2();
	return 0;
}




/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation                      *
 * Date: Fri, 6 Apr 2001 07:18:48 -0500 (CDT)                            *
 * From: "Srigurunath (Ecap) Chakravarthi" <ecap@MPI-Softtech.Com>       *
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <limits.h>

#include "gm.h"

#define maxDataSize	(16 * 1024 * 1024)
#define ITERS		100
#define MILLION		(1.0e+6)
#define DMAX		(double) (INT_MAX)

int
GetNextSize(int DataSize)
{
	if (DataSize == 0)
		return 4;
	else
		return DataSize * 2;
}

double
GetTimeInSec(struct timeval tv)
{
	return ((double) (tv.tv_sec) + (double) (tv.tv_usec) * 1e-6);
}

void
msti_main(void)
{
	struct gm_port *p;
	unsigned char *buffer;
	gm_status_t status;
	unsigned int times;
	long dataSize;
	struct timeval start, middle, end;
	double start_sec, middle_sec, end_sec;
	double reg_time, dereg_time, total_reg_time = 0, total_dereg_time = 0;
	double min_reg_time = DMAX, min_dereg_time = DMAX;
	double max_reg_time = 0.0, max_dereg_time = 0.0;


	if (gm_open(&p, 0, 2, "gm_time_reg", (enum gm_api_version) GM_API_VERSION)
		!= GM_SUCCESS)
	{
		printf("Couldn't open port 'p1' = %d\n", 2);
		exit(-1);
	}


	printf("Buffer\t\tgm_register_memory\t\tgm_deregister_memory\n");
	printf("Length\t\t[min]  [avg] [max]\t\t[min]  [avg] [max]\n");
	printf("[Bytes]\t\t\t[usec]\t\t\t[usec]\n");
	fflush(stdout);


	for (dataSize = 4096; dataSize <= maxDataSize;
			dataSize = GetNextSize(dataSize)) {

		total_dereg_time = total_reg_time = 0;
		min_reg_time = min_dereg_time = DMAX;
		max_reg_time = max_dereg_time = 0.0;

		/* for (times=0; times < ITERS; times++) { */

		buffer = malloc(maxDataSize);

		if (!buffer) {
			printf("Couldn't allocate a buffer\n");
			gm_close(p);
		}

		for (times = 0; times < maxDataSize / dataSize; times++) {
			gettimeofday(&start, NULL);

			status = gm_register_memory(p, buffer + (times * dataSize), dataSize);

			if (status != GM_SUCCESS) {
				printf("Couldn't register memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&middle, NULL);
			start_sec = GetTimeInSec(start);
			middle_sec = GetTimeInSec(middle);

			reg_time = middle_sec - start_sec;

			/* Record min and max */
			if (reg_time < min_reg_time)
				min_reg_time = reg_time;

			if (reg_time > max_reg_time)
				max_reg_time = reg_time;

			/* Compute total  time */
			total_reg_time += reg_time;
		}


		for (times = 0; times < maxDataSize / dataSize; times++) {
			gettimeofday(&middle, NULL);

			status = gm_deregister_memory(p, buffer + (times * dataSize), dataSize);
			if (status != GM_SUCCESS) {
				printf("Couldn't deregister memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&end, NULL);


			middle_sec = GetTimeInSec(middle);
			end_sec = GetTimeInSec(end);

			dereg_time = end_sec - middle_sec;

			/* Record min and max */
			if (dereg_time < min_dereg_time)
				min_dereg_time = dereg_time;

			if (dereg_time > max_dereg_time)
				max_dereg_time = dereg_time;

			/* Compute total  time */
			total_dereg_time += dereg_time;

		}						/* for ITERS */

		free(buffer);

		printf("%10ld    %8.2f %8.2f %8.2f    %8.2f %8.2f %8.2f\n",
			   dataSize,
			   min_reg_time * MILLION,
			   ((total_reg_time / (maxDataSize / dataSize)) * MILLION),		/* avg */
			   max_reg_time * MILLION,
			   min_dereg_time * MILLION,
			   ((total_dereg_time / (maxDataSize / dataSize)) * MILLION),	/* avg */
			   max_dereg_time * MILLION);

		fflush(stdout);

	}							/* for dataSize */

	gm_close(p);
}




void
msti_main2(void)
{
	struct gm_port *p;
	unsigned char *buffer;
	gm_status_t status;
	unsigned int times;
	long dataSize;
	struct timeval start, middle, end;
	double start_sec, middle_sec, end_sec;
	double reg_time, dereg_time, total_reg_time = 0, total_dereg_time = 0;
	double min_reg_time = DMAX, min_dereg_time = DMAX;
	double max_reg_time = 0.0, max_dereg_time = 0.0;


	if (gm_open(&p, 0, 2, "gm_time_reg", (enum gm_api_version) GM_API_VERSION)
		!= GM_SUCCESS)
	{
		printf("Couldn't open port 'p1' = %d\n", 2);
		exit(-1);
	}


	printf("Buffer\t\tgm_register_memory\t\tgm_deregister_memory\n");
	printf("Length\t\t[min]  [avg] [max]\t\t[min]  [avg] [max]\n");
	printf("[Bytes]\t\t\t[usec]\t\t\t[usec]\n");
	fflush(stdout);


	for (dataSize = 1; dataSize <= maxDataSize;
			dataSize = GetNextSize(dataSize)) {

		total_dereg_time = total_reg_time = 0;
		min_reg_time = min_dereg_time = DMAX;
		max_reg_time = max_dereg_time = 0.0;

		for (times = 0; times < ITERS; times++) {

			buffer = malloc(dataSize);

			if (!buffer) {
				printf("Couldn't allocate a buffer\n");
				gm_close(p);
			}

			gettimeofday(&start, NULL);

			status = gm_register_memory(p, buffer, dataSize);

			if (status != GM_SUCCESS) {
				printf("Couldn't register memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&middle, NULL);

			status = gm_deregister_memory(p, buffer, dataSize);
			if (status != GM_SUCCESS) {
				printf("Couldn't deregister memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&end, NULL);


			start_sec = GetTimeInSec(start);
			middle_sec = GetTimeInSec(middle);
			end_sec = GetTimeInSec(end);

			reg_time = middle_sec - start_sec;
			dereg_time = end_sec - middle_sec;

			/* Record min and max */
			if (reg_time < min_reg_time)
				min_reg_time = reg_time;

			if (dereg_time < min_dereg_time)
				min_dereg_time = dereg_time;

			if (reg_time > max_reg_time)
				max_reg_time = reg_time;

			if (dereg_time > max_dereg_time)
				max_dereg_time = dereg_time;

			/* Compute total  time */
			total_reg_time += reg_time;
			total_dereg_time += dereg_time;

			free(buffer);
		}						/* for ITERS */

		printf("%10ld    %8.2f %8.2f %8.2f    %8.2f %8.2f %8.2f\n",
			   dataSize,
			   min_reg_time * MILLION,
			   ((total_reg_time / ITERS) * MILLION),	/* avg */
			   max_reg_time * MILLION,
			   min_dereg_time * MILLION,
			   ((total_dereg_time / ITERS) * MILLION),	/* avg */
			   max_dereg_time * MILLION);

		fflush(stdout);

	}							/* for dataSize */

	gm_close(p);
}




/*
   This file uses feldy indentation:

   Local Variables:
   tab-width:4
   c-file-style:"bsd"
   End:
 */

==> gm-2.0.8_Linux.tar.gz: gm-2.0.8_Linux/tests/gm_time_registration.c <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation                      *
 * Copyright (c) 1996, 1997 by Myricom, Inc.                             *
 * All rights reserved.  See the file `COPYING' for copyright notice.    *
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "gm.h"


#define BUFFER_LEN	(2 * 1024 * 1024)
#define PORT	2


void msti_main(void);
void msti_main2(void);

#ifndef GM_U64_TMPL
#define GM_U64_TMPL "0x%08x%08x"
#define GM_U64_ARG(x)                                           \
  ((gm_u32_t) (((gm_u64_t) (x)) >> 31 >> 1) & 0xffffffff),      \
  ((gm_u32_t) (((gm_u64_t) (x)            ) & 0xffffffff))
#endif

int
main(int argc, char **argv)
{
	struct gm_port *p;
	gm_u64_t start, stop, start2, middle, total_reg, total_dereg;
	unsigned char *buffer, *startbuffer;
	gm_status_t status;
	unsigned int i, num_pages;
	float total_time, reg_time;
	int page_len = (int) getpagesize();
	int looping = 0;
	GM_PARAMETER_MAY_BE_UNUSED(argc);
	GM_PARAMETER_MAY_BE_UNUSED(argv);

	if (argc > 1) {
		looping = 1;
	}

	if (gm_open(&p, 0, PORT, "gm_time_reg",
				(enum gm_api_version) GM_API_VERSION)
		!= GM_SUCCESS)
	{
		printf("Couldn't open port 'p1' = %d\n", PORT);
		exit(-1);
	}

	startbuffer = gm_malloc(BUFFER_LEN + 2 * page_len);
	if (!startbuffer) {
		printf("Couldn't allocate a buffer\n");
		gm_close(p);
	}

	buffer = (unsigned char *) ((((unsigned long) startbuffer) + (page_len - 1)) & ~(page_len - 1));

	printf("Buffer allocated len=%d   start = %p  buffer = %p\n",
		   BUFFER_LEN, startbuffer, buffer);
	fflush(stdout);

	do {
		if (looping) {
			printf("looping\n");
		}

		printf("Registering\n");
		fflush(stdout);

		start = gm_ticks(p);

		for (i = 0; i < BUFFER_LEN; i += page_len) {
			status = gm_register_memory(p, buffer + i, page_len);
			if (status == GM_SUCCESS) {
				/* OK */
			}
			else {
				printf("Couldn't register memory offset = %d (0x%x)\n", i, i);
				gm_close(p);
				exit(-1);
			}
		}
		num_pages = i;

		middle = gm_ticks(p);

		printf("Deregistering\n");
		fflush(stdout);

		start2 = gm_ticks(p);


		for (i = 0; i < BUFFER_LEN; i += page_len) {
			status = gm_deregister_memory(p, buffer + i, page_len);
			if (status == GM_SUCCESS) {
				/* OK */
			}
			else {
				printf("Couldn't DEregister memory offset = %d (0x%x)\n", i, i);
				gm_close(p);
				exit(-1);
			}
		}

		stop = gm_ticks(p);

		total_reg = middle - start;
		total_dereg = stop - start2;

		num_pages = BUFFER_LEN / page_len;
		total_time = ((float) ((unsigned int) total_reg & (unsigned int) 0xFFFFFFFF)) / 2.0;
		printf("  Register time = "GM_U64_TMPL" ticks (%6.2f seconds)  %8.2f usecs/page\n",
			   GM_U64_ARG (total_reg),
			   total_time / 1000000.0, total_time / (float) num_pages);

		total_time = ((float) ((unsigned int) total_dereg & (unsigned int) 0xFFFFFFFF)) / 2.0;
		printf("Deregister time = "GM_U64_TMPL" ticks (%6.2f seconds)  %8.2f usecs/page\n\n\n",
			   GM_U64_ARG (total_dereg),
			   total_time / 1000000.0, total_time / (float) num_pages);

	} while (looping);

	gm_free(startbuffer);
	gm_close(p);

	printf("\nTimings when we register 16meg, then deregister 16 meg\n");
	msti_main();
	printf("\nTimings when we register size ITER times, then unregister size ITER times\n");
	msti_main2();
	return 0;
}




/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation                      *
 * Date: Fri, 6 Apr 2001 07:18:48 -0500 (CDT)                            *
 * From: "Srigurunath (Ecap) Chakravarthi" <ecap@MPI-Softtech.Com>       *
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <limits.h>

#include "gm.h"

#define maxDataSize	(16 * 1024 * 1024)
#define ITERS		100
#define MILLION		(1.0e+6)
#define DMAX		(double) (INT_MAX)

static int
GetNextSize(int DataSize)
{
	if (DataSize == 0)
		return 4;
	else
		return DataSize * 2;
}

static double
GetTimeInSec(struct timeval tv)
{
	return ((double) (tv.tv_sec) + (double) (tv.tv_usec) * 1e-6);
}

void
msti_main(void)
{
	struct gm_port *p;
	unsigned char *buffer;
	gm_status_t status;
	unsigned int times;
	long dataSize;
	struct timeval start, middle, end;
	double start_sec, middle_sec, end_sec;
	double reg_time, dereg_time, total_reg_time = 0, total_dereg_time = 0;
	double min_reg_time = DMAX, min_dereg_time = DMAX;
	double max_reg_time = 0.0, max_dereg_time = 0.0;


	if (gm_open(&p, 0, 2, "gm_time_reg", (enum gm_api_version) GM_API_VERSION)
		!= GM_SUCCESS)
	{
		printf("Couldn't open port 'p1' = %d\n", 2);
		exit(-1);
	}


	printf("Buffer\t\tgm_register_memory\t\tgm_deregister_memory\n");
	printf("Length\t\t[min]  [avg] [max]\t\t[min]  [avg] [max]\n");
	printf("[Bytes]\t\t\t[usec]\t\t\t[usec]\n");
	fflush(stdout);


	for (dataSize = 4096; dataSize <= maxDataSize;
			dataSize = GetNextSize(dataSize)) {

		total_dereg_time = total_reg_time = 0;
		min_reg_time = min_dereg_time = DMAX;
		max_reg_time = max_dereg_time = 0.0;

		/* for (times=0; times < ITERS; times++) { */

		buffer = gm_malloc(maxDataSize);

		if (!buffer) {
			printf("Couldn't allocate a buffer\n");
			gm_close(p);
		}

		for (times = 0; times < maxDataSize / dataSize; times++) {
			gettimeofday(&start, NULL);

			status = gm_register_memory(p, buffer + (times * dataSize), dataSize);

			if (status != GM_SUCCESS) {
				printf("Couldn't register memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&middle, NULL);
			start_sec = GetTimeInSec(start);
			middle_sec = GetTimeInSec(middle);

			reg_time = middle_sec - start_sec;

			/* Record min and max */
			if (reg_time < min_reg_time)
				min_reg_time = reg_time;

			if (reg_time > max_reg_time)
				max_reg_time = reg_time;

			/* Compute total  time */
			total_reg_time += reg_time;
		}


		for (times = 0; times < maxDataSize / dataSize; times++) {
			gettimeofday(&middle, NULL);

			status = gm_deregister_memory(p, buffer + (times * dataSize), dataSize);
			if (status != GM_SUCCESS) {
				printf("Couldn't deregister memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&end, NULL);


			middle_sec = GetTimeInSec(middle);
			end_sec = GetTimeInSec(end);

			dereg_time = end_sec - middle_sec;

			/* Record min and max */
			if (dereg_time < min_dereg_time)
				min_dereg_time = dereg_time;

			if (dereg_time > max_dereg_time)
				max_dereg_time = dereg_time;

			/* Compute total  time */
			total_dereg_time += dereg_time;

		}						/* for ITERS */

		gm_free(buffer);

		printf("%10ld    %8.2f %8.2f %8.2f    %8.2f %8.2f %8.2f\n",
			   dataSize,
			   min_reg_time * MILLION,
			   ((total_reg_time / (maxDataSize / dataSize)) * MILLION),		/* avg */
			   max_reg_time * MILLION,
			   min_dereg_time * MILLION,
			   ((total_dereg_time / (maxDataSize / dataSize)) * MILLION),	/* avg */
			   max_dereg_time * MILLION);

		fflush(stdout);

	}							/* for dataSize */

	gm_close(p);
}




void
msti_main2(void)
{
	struct gm_port *p;
	unsigned char *buffer;
	gm_status_t status;
	unsigned int times;
	long dataSize;
	struct timeval start, middle, end;
	double start_sec, middle_sec, end_sec;
	double reg_time, dereg_time, total_reg_time = 0, total_dereg_time = 0;
	double min_reg_time = DMAX, min_dereg_time = DMAX;
	double max_reg_time = 0.0, max_dereg_time = 0.0;


	if (gm_open(&p, 0, 2, "gm_time_reg", (enum gm_api_version) GM_API_VERSION)
		!= GM_SUCCESS)
	{
		printf("Couldn't open port 'p1' = %d\n", 2);
		exit(-1);
	}


	printf("Buffer\t\tgm_register_memory\t\tgm_deregister_memory\n");
	printf("Length\t\t[min]  [avg] [max]\t\t[min]  [avg] [max]\n");
	printf("[Bytes]\t\t\t[usec]\t\t\t[usec]\n");
	fflush(stdout);


	for (dataSize = 1; dataSize <= maxDataSize;
			dataSize = GetNextSize(dataSize)) {

		total_dereg_time = total_reg_time = 0;
		min_reg_time = min_dereg_time = DMAX;
		max_reg_time = max_dereg_time = 0.0;

		for (times = 0; times < ITERS; times++) {

			buffer = gm_malloc(dataSize);

			if (!buffer) {
				printf("Couldn't allocate a buffer\n");
				gm_close(p);
			}

			gettimeofday(&start, NULL);

			status = gm_register_memory(p, buffer, dataSize);

			if (status != GM_SUCCESS) {
				printf("Couldn't register memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&middle, NULL);

			status = gm_deregister_memory(p, buffer, dataSize);
			if (status != GM_SUCCESS) {
				printf("Couldn't deregister memory. times=%d. dataSize=%ld\n",
					   times, dataSize);
				gm_close(p);
				exit(-1);
			}

			gettimeofday(&end, NULL);


			start_sec = GetTimeInSec(start);
			middle_sec = GetTimeInSec(middle);
			end_sec = GetTimeInSec(end);

			reg_time = middle_sec - start_sec;
			dereg_time = end_sec - middle_sec;

			/* Record min and max */
			if (reg_time < min_reg_time)
				min_reg_time = reg_time;

			if (dereg_time < min_dereg_time)
				min_dereg_time = dereg_time;

			if (reg_time > max_reg_time)
				max_reg_time = reg_time;

			if (dereg_time > max_dereg_time)
				max_dereg_time = dereg_time;

			/* Compute total  time */
			total_reg_time += reg_time;
			total_dereg_time += dereg_time;

			gm_free(buffer);
		}						/* for ITERS */

		printf("%10ld    %8.2f %8.2f %8.2f    %8.2f %8.2f %8.2f\n",
			   dataSize,
			   min_reg_time * MILLION,
			   ((total_reg_time / ITERS) * MILLION),	/* avg */
			   max_reg_time * MILLION,
			   min_dereg_time * MILLION,
			   ((total_dereg_time / ITERS) * MILLION),	/* avg */
			   max_dereg_time * MILLION);

		fflush(stdout);

	}							/* for dataSize */

	gm_close(p);
}




/*
   This file uses feldy indentation:

   Local Variables:
   tab-width:4
   c-file-style:"bsd"
   End:
 */
